@@ -45,7 +45,7 @@
 #include "oggz_private.h"
 #include "oggz_vector.h"
 
-#include <oggz/oggz_stream.h>
+#include "oggz/oggz_stream.h"
 
 /*#define DEBUG*/
 
@@ -147,11 +147,11 @@ The comment header is decoded as follows:
                                      buf[base+2]=(char)((val)&0xff);
 
 static int
-oggz_comment_validate_byname (const char * name, const char * value)
+oggz_comment_validate_byname (const char * name)
 {
   const char * c;
 
-  if (!name || !value) return 0;
+  if (!name) return 0;
 
   for (c = name; *c; c++) {
     if (*c < 0x20 || *c > 0x7D || *c == 0x3D) {
@@ -172,8 +172,8 @@ oggz_comment_new (const char * name, const char * value)
 {
   OggzComment * comment;
 
-  if (!oggz_comment_validate_byname (name, value)) return NULL;
-  /* Ensures that name != NULL, value != NULL, and validates strings */
+  if (!oggz_comment_validate_byname (name)) return NULL;
+  /* Ensures that name != NULL and contains only valid characters */
 
   comment = oggz_malloc (sizeof (OggzComment));
   if (comment == NULL) return NULL;
@@ -184,32 +184,41 @@ oggz_comment_new (const char * name, const char * value)
     return NULL;
   }
 
-  comment->value = oggz_strdup (value);
-  if (comment->value == NULL) {
-    oggz_free (comment->name);
-    oggz_free (comment);
-    return NULL;
+  if (value) {
+    comment->value = oggz_strdup (value);
+    if (comment->value == NULL) {
+      oggz_free (comment->name);
+      oggz_free (comment);
+      return NULL;
+    }
+  } else {
+    comment->value = NULL;
   }
 
   return comment;
 }
 
-static void
-oggz_comment_free (OggzComment * comment)
+static int
+oggz_comment_free (OggzComment * comment_ptr)
 {
-  if (!comment) return;
+  OggzComment * comment = (OggzComment *)comment_ptr;
+  if (!comment) return 0;
   if (comment->name) oggz_free (comment->name);
   if (comment->value) oggz_free (comment->value);
   oggz_free (comment);
+  return 0;
 }
 
 static int
-oggz_comment_cmp (const OggzComment * comment1, const OggzComment * comment2)
+oggz_comment_cmp (const OggzComment * comment1, const OggzComment * comment2, const void * UNUSED(unused))
 {
   if (comment1 == comment2) return 1;
   if (!comment1 || !comment2) return 0;
 
   if (strcasecmp (comment1->name, comment2->name)) return 0;
+
+  if (!comment1->value && !comment2->value) return 1;
+  if (!comment1->value || !comment2->value) return 0;
   if (strcmp (comment1->value, comment2->value)) return 0;
 
   return 1;
@@ -302,7 +311,7 @@ oggz_comment_first_byname (OGGZ * oggz, long serialno, char * name)
 
   if (name == NULL) return oggz_vector_nth_p (stream->comments, 0);
 
-  if (!oggz_comment_validate_byname (name, ""))
+  if (!oggz_comment_validate_byname (name))
     return NULL;
 
   for (i = 0; i < oggz_vector_size (stream->comments); i++) {
@@ -354,7 +363,31 @@ oggz_comment_next_byname (OGGZ * oggz, long serialno,
   return NULL;
 }
 
-#define _oggz_comment_add(f,c) oggz_vector_insert_p ((f)->comments, (c))
+static OggzComment *
+_oggz_comment_add_byname (oggz_stream_t * stream, const char * name, const char * value)
+{
+  OggzComment * comment, * new_comment;
+  int i;
+
+  /* Check that the same name=value pair is not already present */
+  for (i = 0; i < oggz_vector_size (stream->comments); i++) {
+    comment = (OggzComment *) oggz_vector_nth_p (stream->comments, i);
+    if (comment->name && !strcasecmp (name, comment->name)) {
+      if (comment->value == NULL) {
+        if (value == NULL) return comment;
+      } else if ((value && !strcmp (value, comment->value)) || 
+                 (value == NULL && comment->value == NULL)) {
+        return comment;
+      }
+    }
+  }
+
+  /* Allocate new comment and insert it */
+  if ((new_comment = oggz_comment_new (name, value)) == NULL)
+    return NULL;
+
+  return oggz_vector_insert_p (stream->comments, new_comment);
+}
 
 int
 oggz_comment_add (OGGZ * oggz, long serialno, const OggzComment * comment)
@@ -373,13 +406,10 @@ oggz_comment_add (OGGZ * oggz, long serialno, const OggzComment * comment)
   if (oggz->flags & OGGZ_WRITE) {
     if (OGGZ_CONFIG_WRITE) {
 
-      if (!oggz_comment_validate_byname (comment->name, comment->value))
+      if (!oggz_comment_validate_byname (comment->name))
         return OGGZ_ERR_COMMENT_INVALID;
 
-      if ((new_comment = oggz_comment_new (comment->name, comment->value)) == NULL)
-        return OGGZ_ERR_OUT_OF_MEMORY;
-
-      if (_oggz_comment_add (stream, new_comment) == NULL)
+      if (_oggz_comment_add_byname (stream, comment->name, comment->value) == NULL)
         return OGGZ_ERR_OUT_OF_MEMORY;
 
       return 0;
@@ -409,13 +439,10 @@ oggz_comment_add_byname (OGGZ * oggz, long serialno,
   if (oggz->flags & OGGZ_WRITE) {
     if (OGGZ_CONFIG_WRITE) {
 
-      if (!oggz_comment_validate_byname (name, value))
+      if (!oggz_comment_validate_byname (name))
         return OGGZ_ERR_COMMENT_INVALID;
 
-      if ((new_comment = oggz_comment_new (name, value)) == NULL)
-        return OGGZ_ERR_OUT_OF_MEMORY;
-
-      if (_oggz_comment_add (stream, new_comment) == NULL)
+      if (_oggz_comment_add_byname (stream, name, value) == NULL)
         return OGGZ_ERR_OUT_OF_MEMORY;
 
       return 0;
@@ -570,8 +597,10 @@ oggz_comments_decode (OGGZ * oggz, long serialno,
      if ((nvalue = oggz_strdup_len (c, len)) == NULL)
        return OGGZ_ERR_OUT_OF_MEMORY;
 
-     if (_oggz_comment_set_vendor (oggz, serialno, nvalue) == OGGZ_ERR_OUT_OF_MEMORY)
+     if (_oggz_comment_set_vendor (oggz, serialno, nvalue) == OGGZ_ERR_OUT_OF_MEMORY) {
+       oggz_free (nvalue);
        return OGGZ_ERR_OUT_OF_MEMORY;
+     }
 
      oggz_free (nvalue);
    }
@@ -595,40 +624,58 @@ oggz_comments_decode (OGGZ * oggz, long serialno,
       c+=4;
       if (len>(size_t)(end-c)) return -1;
 
+      n = 0;
       name = c;
       value = oggz_index_len (c, '=', len);
       if (value) {
          *value = '\0';
          value++;
-
          n = c+len - value;
+      }
+
+      /* Comment names with zero length (aka "") are invalid */
+      if (0 == name[0]) return OGGZ_ERR_COMMENT_INVALID;
+
+      if (n != 0) {
          if ((nvalue = oggz_strdup_len (value, n)) == NULL)
            return OGGZ_ERR_OUT_OF_MEMORY;
 
 #ifdef DEBUG
-         printf ("oggz_comments_decode: %s -> %s (length %d)\n",
-         name, nvalue, n);
+         printf ("oggz_comments_decode: [%d] %s -> %s (length %d)\n",
+         i, name, nvalue, n);
 #endif
-         if ((comment = oggz_comment_new (name, nvalue)) == NULL)
-           return OGGZ_ERR_OUT_OF_MEMORY;
-
-         if (_oggz_comment_add (stream, comment) == NULL)
+         if (_oggz_comment_add_byname (stream, name, nvalue) == NULL) {
+           oggz_free (nvalue);
            return OGGZ_ERR_OUT_OF_MEMORY;
+	 }
 
          oggz_free (nvalue);
       } else {
+        /* For the case of a comment which is not in key=value form,
+         * duplicate exactly the length of the comment, as it is
+         * not NUL-terminated. In the case of the last comment of the
+         * packet, it will be followed immediately by a framing bit.
+         */
          if ((nvalue = oggz_strdup_len (name, len)) == NULL)
            return OGGZ_ERR_OUT_OF_MEMORY;
 
-         if ((comment = oggz_comment_new (nvalue, NULL)) == NULL)
-           return OGGZ_ERR_OUT_OF_MEMORY;
+#ifdef DEBUG
+         printf ("oggz_comments_decode: [%d] %s (no value) (length %zu)\n",
+         i, nvalue, len);
+#endif
 
-         if (_oggz_comment_add (stream, comment) == NULL)
+         if (_oggz_comment_add_byname (stream, nvalue, NULL) == NULL) {
+           oggz_free (nvalue);
            return OGGZ_ERR_OUT_OF_MEMORY;
+	 }
 
          oggz_free (nvalue);
       }
 
+      /* If we nulled out an equal sign, put it back. */
+      if (value)
+         value[-1] = '=';
+
       c+=len;
    }
 
@@ -681,7 +728,8 @@ oggz_comments_encode (OGGZ * oggz, long serialno,
   if (accum_length (&actual_length, 4 + vendor_length) == 0)
     return 0;
 #ifdef DEBUG
-  printf ("oggz_comments_encode: vendor = %s\n", stream->vendor);
+  printf ("oggz_comments_encode: vendor = %s\n",
+	  stream->vendor ? stream->vendor : "(null)");
 #endif
 
   /* user comment list length */
@@ -795,6 +843,10 @@ oggz_comment_generate(OGGZ * oggz, long serialno,
     {0x04, 0x00, 0x00, 0x00};
   const unsigned char preamble_kate[9] =
     {0x81, 0x6b, 0x61, 0x74, 0x65, 0x00, 0x00, 0x00, 0x00};
+  const unsigned char preamble_opus[8] =
+    {'O', 'p',  'u', 's', 'T', 'a', 'g', 's'};
+  const unsigned char preamble_vp8[7] =
+    {0x4f, 'V',  'P', '8', '0', 002, ' '};
 
 
   switch(packet_type) {
@@ -814,6 +866,14 @@ oggz_comment_generate(OGGZ * oggz, long serialno,
       preamble_length = sizeof preamble_kate;
       preamble = preamble_kate;
       break;
+    case OGGZ_CONTENT_OPUS:
+      preamble_length = sizeof preamble_opus;
+      preamble = preamble_opus;
+      break;
+    case OGGZ_CONTENT_VP8:
+      preamble_length = sizeof preamble_vp8;
+      preamble = preamble_vp8;
+      break;
     case OGGZ_CONTENT_PCM:
     case OGGZ_CONTENT_SPEEX:
       preamble_length = 0;
@@ -874,7 +934,7 @@ oggz_comment_generate(OGGZ * oggz, long serialno,
   return c_packet;
 }
 
-/* In Flac, OggPCM, Speex, Theora Vorbis, and Kate the comment packet will
+/* In Flac, OggPCM, Speex, Theora, Vorbis, Kate, Opus and VP8, the comment packet will
    be second in the stream, i.e. packetno=1, and it will have granulepos=0 */
 ogg_packet *
 oggz_comments_generate(OGGZ * oggz, long serialno,
