@@ -328,4 +328,9 @@ int oggz_io_seek (OGGZ * oggz, long offset, int whence);
 long oggz_io_tell (OGGZ * oggz);
 int oggz_io_flush (OGGZ * oggz);
 
+/* oggz_read */
+#if OGGZ_CONFIG_READ
+OggzDListIterResponse oggz_read_free_pbuffers(void *elem);
+#endif /* OGGZ_CONFIG_READ */
+
 #endif /* __OGGZ_PRIVATE_H__ */
