@@ -139,6 +139,7 @@ static void
 oggz_reset_streams (OGGZ * oggz)
 {
   oggz_vector_foreach (oggz->streams, oggz_stream_reset);
+  oggz->x.reader.current_page_bytes = 0;
 }
 
 static long
@@ -371,7 +372,7 @@ oggz_get_prev_start_page (OGGZ * oggz, ogg_page * og,
 
 #ifdef DEBUG
     printf ("get_prev_start_page: [C] offset_at: @%" PRI_OGGZ_OFF_T "d\t"
-	    "found_offset: @%" PRI_OGGZ_OFF_T "d\tunit_at: %lld\n",
+	    "found_offset: @%" PRI_OGGZ_OFF_T "d\tunit_at: %" PRId64 "\n",
 	    offset_at, found_offset, unit_at);
 #endif
 
@@ -455,13 +456,13 @@ oggz_scan_for_page (OGGZ * oggz, ogg_page * og, ogg_int64_t unit_target,
 
     if (unit_at < unit_target) {
 #ifdef DEBUG
-      printf (" scan: (%lld) < (%lld)\n", unit_at, unit_target);
+      printf (" scan: (%" PRId64 ") < (%" PRId64 ")\n", unit_at, unit_target);
 #endif
       offset_prev = offset_next;
       offset_begin = offset_next+1;
     } else if (unit_at > unit_target) {
 #ifdef DEBUG
-      printf (" scan: (%lld) > (%lld)\n", unit_at, unit_target);
+      printf (" scan: (%" PRId64 ") > (%" PRId64 ")\n", unit_at, unit_target);
 #endif
 #if 0
       /* hole ? */
@@ -479,13 +480,15 @@ oggz_scan_for_page (OGGZ * oggz, ogg_page * og, ogg_int64_t unit_target,
 #else
       do {
         offset_at = oggz_get_prev_start_page(oggz, og, &granule_at, &serialno);
+        if (offset_at < 0)
+          break;
         unit_at = oggz_get_unit(oggz, serialno, granule_at);
       } while (unit_at > unit_target);
       return offset_at;
 #endif
     } else if (unit_at == unit_target) {
 #ifdef DEBUG
-      printf (" scan: (%lld) == (%lld)\n", unit_at, unit_target);
+      printf (" scan: (%" PRId64 ") == (%" PRId64 ")\n", unit_at, unit_target);
 #endif
       break;
     }
@@ -517,7 +520,7 @@ guess (ogg_int64_t unit_at, ogg_int64_t unit_target,
   }
 
 #ifdef DEBUG
-  printf ("oggz_seek::guess: guess_ratio %lld = (%lld - %lld) / (%lld - %lld)\n",
+  printf ("oggz_seek::guess: guess_ratio %" PRId64 " = (%" PRId64 " - %" PRId64 ") / (%" PRId64 " - %" PRId64 ")\n",
 	  guess_ratio, unit_target, unit_begin, unit_at, unit_begin);
 #endif
   
@@ -706,7 +709,7 @@ oggz_bounded_seek_set (OGGZ * oggz,
     hit_eof = 0;
 
 #ifdef DEBUG
-    printf ("oggz_bounded_seek_set: [A] want u%lld: (u%lld - u%lld) [@%" PRI_OGGZ_OFF_T "d - @%" PRI_OGGZ_OFF_T "d]\n",
+    printf ("oggz_bounded_seek_set: [A] want u%" PRId64 ": (u%" PRId64 " - u%" PRId64 ") [@%" PRI_OGGZ_OFF_T "d - @%" PRI_OGGZ_OFF_T "d]\n",
 	    unit_target, unit_begin, unit_end, offset_begin, offset_end);
 #endif
 
@@ -740,7 +743,7 @@ oggz_bounded_seek_set (OGGZ * oggz,
     if (unit_at == unit_last_iter) break;
 
 #ifdef DEBUG
-    printf ("oggz_bounded_seek_set: [D] want u%lld, got page u%lld @%" PRI_OGGZ_OFF_T "d g%lld\n",
+    printf ("oggz_bounded_seek_set: [D] want u%" PRId64 ", got page u%" PRId64 " @%" PRI_OGGZ_OFF_T "d g%" PRId64 "\n",
 	    unit_target, unit_at, offset_at, granule_at);
 #endif
 
@@ -771,7 +774,7 @@ oggz_bounded_seek_set (OGGZ * oggz,
   if (offset_at == -1) return -1;
 
 #ifdef DEBUG
-  printf ("oggz_bounded_seek_set: FOUND (%lld)\n", unit_at);
+  printf ("oggz_bounded_seek_set: FOUND (%" PRId64 ")\n", unit_at);
 #endif
 
   return (long)reader->current_unit;
@@ -803,7 +806,7 @@ oggz_seek_end (OGGZ * oggz, ogg_int64_t unit_offset)
   unit_end = oggz_get_unit (oggz, serialno, granulepos);
 
 #ifdef DEBUG
-  printf ("*** oggz_seek_end: found packet (%lld) at @%" PRI_OGGZ_OFF_T "d [%lld]\n",
+  printf ("*** oggz_seek_end: found packet (%" PRId64 ") at @%" PRI_OGGZ_OFF_T "d [%" PRId64 "]\n",
 	  unit_end, offset_end, granulepos);
 #endif
 
@@ -886,13 +889,13 @@ oggz_seek_units (OGGZ * oggz, ogg_int64_t units, int whence)
 }
 
 long
-oggz_seek_byorder (OGGZ * oggz, void * target)
+oggz_seek_byorder (OGGZ * UNUSED(oggz), void * UNUSED(target))
 {
   return -1;
 }
 
 long
-oggz_seek_packets (OGGZ * oggz, long serialno, long packets, int whence)
+oggz_seek_packets (OGGZ * UNUSED(oggz), long UNUSED(serialno), long UNUSED(packets), int UNUSED(whence))
 {
   return -1;
 }
