@@ -43,20 +43,18 @@ typedef int (*OggzFunc1) (void * data, void * arg);
 typedef int (*OggzFindFunc) (void * data, long serialno);
 typedef int (*OggzCmpFunc) (const void * a, const void * b, void * user_data);
 
-typedef struct _OggzVector OggzVector;
-
 typedef union {
   void * p;
   long l;
 } oggz_data_t;
 
-struct _OggzVector {
+typedef struct {
   int max_elements;
   int nr_elements;
   oggz_data_t * data;
   OggzCmpFunc compare;
   void * compare_user_data;
-};
+} OggzVector;
 
 /*
  * A vector of void * or long; iff it's a vector of void * objects, it
@@ -78,6 +76,7 @@ oggz_vector_new (void)
   OggzVector * vector;
 
   vector = oggz_malloc (sizeof (OggzVector));
+  if (vector == NULL) return NULL;
 
   vector->max_elements = 0;
   vector->nr_elements = 0;
@@ -266,7 +265,6 @@ oggz_vector_grow (OggzVector * vector)
 
     if (new_elements == NULL) {
       vector->nr_elements--;
-      vector->data = NULL;
       return NULL;
     }
 
@@ -359,8 +357,7 @@ oggz_vector_remove_nth (OggzVector * vector, int n)
         oggz_realloc (vector->data,
         (size_t)new_max_elements * sizeof (oggz_data_t));
 
-      if (new_elements == NULL)
-      {
+      if (new_elements == NULL) {
         vector->data = NULL;
         return NULL;
       }
@@ -405,52 +402,13 @@ void *
 oggz_vector_pop (OggzVector * vector)
 {
   void * data;
-#if 0
-  void * new_elements;
-  int new_max_elements;
-#endif
 
-  if (!vector || vector->data == NULL) return NULL;
+  if (vector == NULL || vector->data == NULL) return NULL;
 
   data = vector->data[0].p;
 
-#if 0
-  vector->nr_elements--;
-
-  if (vector->nr_elements == 0) {
-    oggz_vector_clear (vector);
-  } else {
-#if 0
-    memmove (vector->data, &vector->data[1],
-	     vector->nr_elements * sizeof (void *));
-#else
-    {
-      int i;
-      for (i = 0; i < vector->nr_elements; i++) {
-	vector->data[i].p = vector->data[i+1].p;
-      }
-    }
-#endif
-    if (vector->nr_elements < vector->max_elements/2) {
-      new_max_elements = vector->max_elements/2;
-
-      new_elements =
-        oggz_realloc (vector->data,
-        (size_t)new_max_elements * sizeof (oggz_data_t));
-
-      if (new_elements != NULL) {
-        vector->max_elements = new_max_elements;
-        vector->data = new_elements;
-      }
-    }
-
-  }
-#else
-
   oggz_vector_remove_nth (vector, 0);
 
-#endif
-
   return data;
 
 }
