@@ -463,7 +463,8 @@ write_accum (OCState * state)
   OCPageAccum * pa;
   OggzTable * candidates;
   long serialno, min_serialno;
-  int i, ntracks, ncandidates=0, remaining=0, min_i, cn, min_cn;
+  int i, ntracks, ncandidates=0, remaining=0, min_i;
+  ptrdiff_t cn, min_cn;
   ogg_page * og, * min_og;
   double min_time;
 
@@ -503,7 +504,7 @@ write_accum (OCState * state)
     min_og = NULL;
     min_serialno = -1;
     for (i=0; i < ncandidates; i++) {
-      cn = ((int) oggz_table_nth (candidates, i, &serialno)) - CN_OFFSET;
+      cn = ((ptrdiff_t) oggz_table_nth (candidates, i, &serialno)) - CN_OFFSET;
       ts = oggz_table_lookup (state->tracks, serialno);
       if (ts && ts->page_accum) {
         if (cn < oggz_table_size (ts->page_accum)) {
@@ -744,6 +745,59 @@ read_dirac (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
   return OGGZ_CONTINUE;
 }
 
+static int
+read_vp8 (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
+{
+  OCState * state = (OCState *)user_data;
+  OCTrackState * ts;
+  OCPageAccum * pa;
+  double page_time;
+  ogg_int64_t granulepos, pts, dist, keyframe;
+  int accum_size;
+
+  page_time = oggz_tell_units (oggz) / 1000.0;
+
+  ts = oggz_table_lookup (state->tracks, serialno);
+  accum_size = oggz_table_size (ts->page_accum);
+
+  if (page_time >= state->start) {
+    /* Glue in fisbones, write out accumulated pages */
+    chop_glue (state, oggz);
+
+    /* Switch to the plain page reader */
+    oggz_set_read_page (oggz, serialno, read_plain, state);
+    return read_plain (oggz, og, serialno, user_data);
+  } /* else { ... */
+
+  granulepos = ogg_page_granulepos (OGG_PAGE_CONST(og));
+  if (granulepos != -1) {
+    pts = granulepos >> 32;
+    dist = (granulepos >> 3) & 0x7ffffff;
+    keyframe = pts - dist;
+
+    if (keyframe != ts->prev_keyframe) {
+      if (ogg_page_continued(OGG_PAGE_CONST(og))) {
+        /* If this new-keyframe page is continued, advance the page accumulator,
+         * ie. recover earlier pages from this new GOP */
+        accum_size = track_state_advance_page_accum (ts);
+      } else {
+        /* Otherwise, just clear the page accumulator */
+        track_state_remove_page_accum (ts);
+        accum_size = 0;
+      }
+
+      /* Record this as prev_keyframe */
+      ts->prev_keyframe = keyframe;
+    }
+  }
+
+  /* Add a copy of this to the page accumulator */
+  pa = page_accum_new (og, page_time);
+  oggz_table_insert (ts->page_accum, accum_size, pa);
+
+  return OGGZ_CONTINUE;
+}
+
 /*
  * OggzReadPageCallback read_headers
  *
@@ -781,6 +835,8 @@ read_headers (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
         oggz_set_read_page (oggz, serialno, read_plain, state);
       } else if (content_type == OGGZ_CONTENT_DIRAC) {
         oggz_set_read_page (oggz, serialno, read_dirac, state);
+      } else if (content_type == OGGZ_CONTENT_VP8) {
+        oggz_set_read_page (oggz, serialno, read_vp8, state);
       } else {
         oggz_set_read_page (oggz, serialno, read_gs, state);
       }
