@@ -40,6 +40,7 @@
 #include <errno.h>
 
 #include "oggz/oggz.h"
+#include "oggz_macros.h"
 #include "oggz_tools.h"
 
 #include "skeleton.h"
@@ -148,15 +149,21 @@ gp_to_granule (OGGZ * oggz, long serialno, ogg_int64_t granulepos)
 {
   int granuleshift;
   ogg_int64_t iframe, pframe, granule;
+  OggzStreamContent content;
 
   granuleshift = oggz_get_granuleshift (oggz, serialno);
+  content = oggz_stream_get_content (oggz, serialno);
 
-  iframe = granulepos >> granuleshift;
-  pframe = granulepos - (iframe << granuleshift);
-  granule = iframe + pframe;
+  if (content == OGGZ_CONTENT_VP8) {
+    granule = granulepos >> granuleshift;
+  } else {
+    iframe = granulepos >> granuleshift;
+    pframe = granulepos - (iframe << granuleshift);
+    granule = iframe + pframe;
 
-  if (oggz_stream_get_content (oggz, serialno) == OGGZ_CONTENT_DIRAC)
-    granule >>= 9;
+    if (content == OGGZ_CONTENT_DIRAC)
+      granule >>= 9;
+  }
 
   return granule;
 }
@@ -237,7 +244,7 @@ oi_bitrate (long bytes, ogg_int64_t ms)
 }
 
 static void
-oi_stats_print (OI_Info * info, OI_Stats * stats, char * label)
+oi_stats_print (OI_Info * UNUSED(info), OI_Stats * stats, char * label)
 {
   printf ("\t%s-Length-Maximum: ", label);
   ot_fprint_bytes (stdout, stats->length_max);
@@ -301,7 +308,7 @@ ot_fisbone_print(OI_Info * info, OI_TrackInfo *oit) {
       if (token == NULL)
         break;
       *token = '\0';
-      printf("\t %s", token);
+      printf("\t %s\n", messages);
 
       token++;
       if (*token == '\n')
@@ -375,7 +382,7 @@ oi_stats_average (OI_Stats * stats)
 }
 
 static int
-oit_calc_average (OI_Info * info, OI_TrackInfo * oit, long serialno)
+oit_calc_average (OI_Info * UNUSED(info), OI_TrackInfo * oit, long UNUSED(serialno))
 {
   oi_stats_average (&oit->pages);
   oi_stats_average (&oit->packets);
@@ -397,7 +404,7 @@ oi_stats_stddev (OI_Stats * stats)
 }
 
 static int
-oit_calc_stddev (OI_Info * info, OI_TrackInfo * oit, long serialno)
+oit_calc_stddev (OI_Info * UNUSED(info), OI_TrackInfo * oit, long UNUSED(serialno))
 {
   oi_stats_stddev (&oit->pages);
   oi_stats_stddev (&oit->packets);
@@ -441,7 +448,7 @@ read_page_pass1 (OGGZ * oggz, const ogg_page * og, long serialno, void * user_da
 }
 
 static int
-read_page_pass2 (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
+read_page_pass2 (OGGZ * UNUSED(oggz), const ogg_page * og, long serialno, void * user_data)
 {
   OI_Info * info = (OI_Info *)user_data;
   OI_TrackInfo * oit;
@@ -458,7 +465,7 @@ read_page_pass2 (OGGZ * oggz, const ogg_page * og, long serialno, void * user_da
 }
 
 static int
-read_packet_pass1 (OGGZ * oggz, oggz_packet * zp, long serialno,
+read_packet_pass1 (OGGZ * UNUSED(oggz), oggz_packet * zp, long serialno,
 		   void * user_data)
 {
   OI_Info * info = (OI_Info *)user_data;
@@ -500,7 +507,7 @@ read_packet_pass1 (OGGZ * oggz, oggz_packet * zp, long serialno,
 }
 
 static int
-read_packet_pass2 (OGGZ * oggz, oggz_packet * zp, long serialno,
+read_packet_pass2 (OGGZ * UNUSED(oggz), oggz_packet * zp, long serialno,
 		   void * user_data)
 {
   OI_Info * info = (OI_Info *)user_data;
@@ -571,7 +578,7 @@ oi_pass2 (OGGZ * oggz, OI_Info * info)
 }
 
 static int
-oit_delete (OI_Info * info, OI_TrackInfo * oit, long serialno)
+oit_delete (OI_Info * UNUSED(info), OI_TrackInfo * oit, long UNUSED(serialno))
 {
   if (oit->codec_info) {
     if (oit->has_fisbone)
