@@ -47,6 +47,7 @@
 #endif
 
 #include "oggz/oggz.h"
+#include "oggz_macros.h"
 #include "oggz_tools.h"
 
 #define READ_SIZE 4096
@@ -185,7 +186,7 @@ omdata_delete (OMData * omdata)
 }
 
 static int
-read_page (OGGZ * oggz, const ogg_page * og, long serialno, void * user_data)
+read_page (OGGZ * UNUSED(oggz), const ogg_page * og, long UNUSED(serialno), void * user_data)
 {
   OMInput * input = (OMInput *) user_data;
 
